diff mbox

[24/26] libcacard: move atr setting from macro to function

Message ID 1363612272-13713-25-git-send-email-alevy@redhat.com
State New
Headers show

Commit Message

Alon Levy March 18, 2013, 1:11 p.m. UTC
Only because qemu's checkpatch complains about it.

Signed-off-by: Alon Levy <alevy@redhat.com>
---
 Makefile.objs              |  1 +
 libcacard/vcard_emul_nss.c | 11 ++++++++---
 libcacard/vcardt.c         | 40 ++++++++++++++++++++++++++++++++++++++++
 libcacard/vcardt.h         | 15 ++-------------
 4 files changed, 51 insertions(+), 16 deletions(-)
 create mode 100644 libcacard/vcardt.c

Comments

Marc-André Lureau March 22, 2013, 2:22 p.m. UTC | #1
Hi,

review below

On Mon, Mar 18, 2013 at 2:11 PM, Alon Levy <alevy@redhat.com> wrote:
> Only because qemu's checkpatch complains about it.
>
> Signed-off-by: Alon Levy <alevy@redhat.com>
> ---
>  Makefile.objs              |  1 +
>  libcacard/vcard_emul_nss.c | 11 ++++++++---
>  libcacard/vcardt.c         | 40 ++++++++++++++++++++++++++++++++++++++++
>  libcacard/vcardt.h         | 15 ++-------------
>  4 files changed, 51 insertions(+), 16 deletions(-)
>  create mode 100644 libcacard/vcardt.c
>
> diff --git a/Makefile.objs b/Makefile.objs
> index f99841c..6d47567 100644
> --- a/Makefile.objs
> +++ b/Makefile.objs
> @@ -41,6 +41,7 @@ libcacard-y += libcacard/vcard.o libcacard/vreader.o
>  libcacard-y += libcacard/vcard_emul_nss.o
>  libcacard-y += libcacard/vcard_emul_type.o
>  libcacard-y += libcacard/card_7816.o
> +libcacard-y += libcacard/vcardt.o
>
>  ######################################################################
>  # Target independent part of system emulation. The long term path is to
> diff --git a/libcacard/vcard_emul_nss.c b/libcacard/vcard_emul_nss.c
> index 21d4689..6bad0b9 100644
> --- a/libcacard/vcard_emul_nss.c
> +++ b/libcacard/vcard_emul_nss.c
> @@ -519,18 +519,23 @@ vcard_emul_reader_get_slot(VReader *vreader)
>  }
>
>  /*
> - *  Card ATR's map to physical cards. VCARD_ATR_PREFIX will set appropriate
> + *  Card ATR's map to physical cards. vcard_alloc_atr will set appropriate
>   *  historical bytes for any software emulated card. The remaining bytes can be
>   *  used to indicate the actual emulator
>   */
> -static const unsigned char nss_atr[] = { VCARD_ATR_PREFIX(3), 'N', 'S', 'S' };
> +static unsigned char *nss_atr;
> +static int nss_atr_len;
>
>  void
>  vcard_emul_get_atr(VCard *card, unsigned char *atr, int *atr_len)
>  {
> -    int len = MIN(sizeof(nss_atr), *atr_len);
> +    int len;
>      assert(atr != NULL);
>
> +    if (nss_atr == NULL) {
> +        nss_atr = vcard_alloc_atr("NSS", &nss_atr_len);
> +    }
> +    len = MIN(nss_atr_len, *atr_len);
>      memcpy(atr, nss_atr, len);
>      *atr_len = len;
>  }
> diff --git a/libcacard/vcardt.c b/libcacard/vcardt.c
> new file mode 100644
> index 0000000..f64c343
> --- /dev/null
> +++ b/libcacard/vcardt.c
> @@ -0,0 +1,40 @@
> +#include <stdlib.h>
> +#include <string.h>
> +#include <glib.h>
> +
> +#include "libcacard/vcardt.h"
> +
> +/* create an ATR with appropriate historical bytes */
> +#define ATR_TS_DIRECT_CONVENTION 0x3b
> +#define ATR_TA_PRESENT 0x10
> +#define ATR_TB_PRESENT 0x20
> +#define ATR_TC_PRESENT 0x40
> +#define ATR_TD_PRESENT 0x80
> +
> +unsigned char *vcard_alloc_atr(const char *postfix, int *atr_len)
> +{
> +    int postfix_len;
> +    const char prefix[] = "VCARD_";
> +    const int prefix_len = strlen(prefix);

or sizeof(prefix) -1

> +    int total_len;
> +    unsigned char *atr;
> +
> +    if (postfix == NULL) {

I would make postfix mandatory.

> +        postfix_len = 0;
> +    } else {
> +        postfix_len = strlen(postfix);
> +    }
> +    total_len = 3 + prefix_len + postfix_len;
> +    atr = g_malloc(total_len);
> +    atr[0] = ATR_TS_DIRECT_CONVENTION;
> +    atr[1] = ATR_TD_PRESENT + prefix_len + postfix_len;
> +    atr[2] = 0x00;
> +    memcpy(&atr[3], prefix, prefix_len);
> +    if (postfix) {
> +        memcpy(&atr[3 + prefix_len], postfix, postfix_len);
> +    }
> +    if (atr_len) {
> +        *atr_len = total_len;
> +    }
> +    return atr;
> +}
> diff --git a/libcacard/vcardt.h b/libcacard/vcardt.h
> index 3b9a619..e8e651f 100644
> --- a/libcacard/vcardt.h
> +++ b/libcacard/vcardt.h
> @@ -25,19 +25,6 @@ typedef struct VCardEmulStruct VCardEmul;
>
>  #define MAX_CHANNEL 4
>
> -/* create an ATR with appropriate historical bytes */
> -#define TS_DIRECT_CONVENTION 0x3b
> -#define TA_PRESENT 0x10
> -#define TB_PRESENT 0x20
> -#define TC_PRESENT 0x40
> -#define TD_PRESENT 0x80
> -
> -#define VCARD_ATR_PREFIX(size) \
> -    TS_DIRECT_CONVENTION, \
> -    TD_PRESENT + (6 + size), \
> -    0x00, \
> -    'V', 'C', 'A', 'R', 'D', '_'
> -
>  typedef enum {
>      VCARD_DONE,
>      VCARD_NEXT,
> @@ -69,4 +56,6 @@ struct VCardBufferResponseStruct {
>      int len;
>  };
>
> +unsigned char *vcard_alloc_atr(const char *postfix, int *atr_len);
> +

If the function is exposed in public header, it should also be
exported (in libcacard.syms), but I am not sure it's useful for
libcacard user.
Alon Levy March 22, 2013, 7:33 p.m. UTC | #2
> Hi,
> 
> review below
> 
> On Mon, Mar 18, 2013 at 2:11 PM, Alon Levy <alevy@redhat.com> wrote:
> > Only because qemu's checkpatch complains about it.
> >
> > Signed-off-by: Alon Levy <alevy@redhat.com>
> > ---
> >  Makefile.objs              |  1 +
> >  libcacard/vcard_emul_nss.c | 11 ++++++++---
> >  libcacard/vcardt.c         | 40
> >  ++++++++++++++++++++++++++++++++++++++++
> >  libcacard/vcardt.h         | 15 ++-------------
> >  4 files changed, 51 insertions(+), 16 deletions(-)
> >  create mode 100644 libcacard/vcardt.c
> >
> > diff --git a/Makefile.objs b/Makefile.objs
> > index f99841c..6d47567 100644
> > --- a/Makefile.objs
> > +++ b/Makefile.objs
> > @@ -41,6 +41,7 @@ libcacard-y += libcacard/vcard.o
> > libcacard/vreader.o
> >  libcacard-y += libcacard/vcard_emul_nss.o
> >  libcacard-y += libcacard/vcard_emul_type.o
> >  libcacard-y += libcacard/card_7816.o
> > +libcacard-y += libcacard/vcardt.o
> >
> >  ######################################################################
> >  # Target independent part of system emulation. The long term path
> >  is to
> > diff --git a/libcacard/vcard_emul_nss.c
> > b/libcacard/vcard_emul_nss.c
> > index 21d4689..6bad0b9 100644
> > --- a/libcacard/vcard_emul_nss.c
> > +++ b/libcacard/vcard_emul_nss.c
> > @@ -519,18 +519,23 @@ vcard_emul_reader_get_slot(VReader *vreader)
> >  }
> >
> >  /*
> > - *  Card ATR's map to physical cards. VCARD_ATR_PREFIX will set
> > appropriate
> > + *  Card ATR's map to physical cards. vcard_alloc_atr will set
> > appropriate
> >   *  historical bytes for any software emulated card. The remaining
> >   bytes can be
> >   *  used to indicate the actual emulator
> >   */
> > -static const unsigned char nss_atr[] = { VCARD_ATR_PREFIX(3), 'N',
> > 'S', 'S' };
> > +static unsigned char *nss_atr;
> > +static int nss_atr_len;
> >
> >  void
> >  vcard_emul_get_atr(VCard *card, unsigned char *atr, int *atr_len)
> >  {
> > -    int len = MIN(sizeof(nss_atr), *atr_len);
> > +    int len;
> >      assert(atr != NULL);
> >
> > +    if (nss_atr == NULL) {
> > +        nss_atr = vcard_alloc_atr("NSS", &nss_atr_len);
> > +    }
> > +    len = MIN(nss_atr_len, *atr_len);
> >      memcpy(atr, nss_atr, len);
> >      *atr_len = len;
> >  }
> > diff --git a/libcacard/vcardt.c b/libcacard/vcardt.c
> > new file mode 100644
> > index 0000000..f64c343
> > --- /dev/null
> > +++ b/libcacard/vcardt.c
> > @@ -0,0 +1,40 @@
> > +#include <stdlib.h>
> > +#include <string.h>
> > +#include <glib.h>
> > +
> > +#include "libcacard/vcardt.h"
> > +
> > +/* create an ATR with appropriate historical bytes */
> > +#define ATR_TS_DIRECT_CONVENTION 0x3b
> > +#define ATR_TA_PRESENT 0x10
> > +#define ATR_TB_PRESENT 0x20
> > +#define ATR_TC_PRESENT 0x40
> > +#define ATR_TD_PRESENT 0x80
> > +
> > +unsigned char *vcard_alloc_atr(const char *postfix, int *atr_len)
> > +{
> > +    int postfix_len;
> > +    const char prefix[] = "VCARD_";
> > +    const int prefix_len = strlen(prefix);
> 
> or sizeof(prefix) -1
> 
> > +    int total_len;
> > +    unsigned char *atr;
> > +
> > +    if (postfix == NULL) {
> 
> I would make postfix mandatory.
> 
> > +        postfix_len = 0;
> > +    } else {
> > +        postfix_len = strlen(postfix);
> > +    }
> > +    total_len = 3 + prefix_len + postfix_len;
> > +    atr = g_malloc(total_len);
> > +    atr[0] = ATR_TS_DIRECT_CONVENTION;
> > +    atr[1] = ATR_TD_PRESENT + prefix_len + postfix_len;
> > +    atr[2] = 0x00;
> > +    memcpy(&atr[3], prefix, prefix_len);
> > +    if (postfix) {
> > +        memcpy(&atr[3 + prefix_len], postfix, postfix_len);
> > +    }
> > +    if (atr_len) {
> > +        *atr_len = total_len;
> > +    }
> > +    return atr;
> > +}
> > diff --git a/libcacard/vcardt.h b/libcacard/vcardt.h
> > index 3b9a619..e8e651f 100644
> > --- a/libcacard/vcardt.h
> > +++ b/libcacard/vcardt.h
> > @@ -25,19 +25,6 @@ typedef struct VCardEmulStruct VCardEmul;
> >
> >  #define MAX_CHANNEL 4
> >
> > -/* create an ATR with appropriate historical bytes */
> > -#define TS_DIRECT_CONVENTION 0x3b
> > -#define TA_PRESENT 0x10
> > -#define TB_PRESENT 0x20
> > -#define TC_PRESENT 0x40
> > -#define TD_PRESENT 0x80
> > -
> > -#define VCARD_ATR_PREFIX(size) \
> > -    TS_DIRECT_CONVENTION, \
> > -    TD_PRESENT + (6 + size), \
> > -    0x00, \
> > -    'V', 'C', 'A', 'R', 'D', '_'
> > -
> >  typedef enum {
> >      VCARD_DONE,
> >      VCARD_NEXT,
> > @@ -69,4 +56,6 @@ struct VCardBufferResponseStruct {
> >      int len;
> >  };
> >
> > +unsigned char *vcard_alloc_atr(const char *postfix, int *atr_len);
> > +
> 
> If the function is exposed in public header, it should also be
> exported (in libcacard.syms), but I am not sure it's useful for
> libcacard user.

It is purely internal, I could add a private header for this.

> 
> 
> --
> Marc-André Lureau
> 
>
diff mbox

Patch

diff --git a/Makefile.objs b/Makefile.objs
index f99841c..6d47567 100644
--- a/Makefile.objs
+++ b/Makefile.objs
@@ -41,6 +41,7 @@  libcacard-y += libcacard/vcard.o libcacard/vreader.o
 libcacard-y += libcacard/vcard_emul_nss.o
 libcacard-y += libcacard/vcard_emul_type.o
 libcacard-y += libcacard/card_7816.o
+libcacard-y += libcacard/vcardt.o
 
 ######################################################################
 # Target independent part of system emulation. The long term path is to
diff --git a/libcacard/vcardt.c b/libcacard/vcardt.c
new file mode 100644
index 0000000..f64c343
--- /dev/null
+++ b/libcacard/vcardt.c
@@ -0,0 +1,40 @@ 
+#include <stdlib.h>
+#include <string.h>
+#include <glib.h>
+
+#include "libcacard/vcardt.h"
+
+/* create an ATR with appropriate historical bytes */
+#define ATR_TS_DIRECT_CONVENTION 0x3b
+#define ATR_TA_PRESENT 0x10
+#define ATR_TB_PRESENT 0x20
+#define ATR_TC_PRESENT 0x40
+#define ATR_TD_PRESENT 0x80
+
+unsigned char *vcard_alloc_atr(const char *postfix, int *atr_len)
+{
+    int postfix_len;
+    const char prefix[] = "VCARD_";
+    const int prefix_len = strlen(prefix);
+    int total_len;
+    unsigned char *atr;
+
+    if (postfix == NULL) {
+        postfix_len = 0;
+    } else {
+        postfix_len = strlen(postfix);
+    }
+    total_len = 3 + prefix_len + postfix_len;
+    atr = g_malloc(total_len);
+    atr[0] = ATR_TS_DIRECT_CONVENTION;
+    atr[1] = ATR_TD_PRESENT + prefix_len + postfix_len;
+    atr[2] = 0x00;
+    memcpy(&atr[3], prefix, prefix_len);
+    if (postfix) {
+        memcpy(&atr[3 + prefix_len], postfix, postfix_len);
+    }
+    if (atr_len) {
+        *atr_len = total_len;
+    }
+    return atr;
+}
diff --git a/libcacard/vcardt.h b/libcacard/vcardt.h
index 3b9a619..e8e651f 100644
--- a/libcacard/vcardt.h
+++ b/libcacard/vcardt.h
@@ -25,19 +25,6 @@  typedef struct VCardEmulStruct VCardEmul;
 
 #define MAX_CHANNEL 4
 
-/* create an ATR with appropriate historical bytes */
-#define TS_DIRECT_CONVENTION 0x3b
-#define TA_PRESENT 0x10
-#define TB_PRESENT 0x20
-#define TC_PRESENT 0x40
-#define TD_PRESENT 0x80
-
-#define VCARD_ATR_PREFIX(size) \
-    TS_DIRECT_CONVENTION, \
-    TD_PRESENT + (6 + size), \
-    0x00, \
-    'V', 'C', 'A', 'R', 'D', '_'
-
 typedef enum {
     VCARD_DONE,
     VCARD_NEXT,
@@ -69,4 +56,6 @@  struct VCardBufferResponseStruct {
     int len;
 };
 
+unsigned char *vcard_alloc_atr(const char *postfix, int *atr_len);
+
 #endif
diff --git a/libcacard/vcard_emul_nss.c b/libcacard/vcard_emul_nss.c
index 21d4689..6bad0b9 100644
--- a/libcacard/vcard_emul_nss.c
+++ b/libcacard/vcard_emul_nss.c
@@ -519,18 +519,23 @@  vcard_emul_reader_get_slot(VReader *vreader)
 }
 
 /*
  *  Card ATR's map to physical cards. VCARD_ATR_PREFIX will set appropriate
  *  historical bytes for any software emulated card. The remaining bytes can be
  *  used to indicate the actual emulator
  */
-static const unsigned char nss_atr[] = { VCARD_ATR_PREFIX(3), 'N', 'S', 'S' };
+static unsigned char *nss_atr;
+static int nss_atr_len;
 
 void
 vcard_emul_get_atr(VCard *card, unsigned char *atr, int *atr_len)
 {
-    int len = MIN(sizeof(nss_atr), *atr_len);
+    int len;
     assert(atr != NULL);
 
+    if (nss_atr == NULL) {
+        nss_atr = vcard_alloc_atr("NSS", &nss_atr_len);
+    }
+    len = MIN(nss_atr_len, *atr_len);
     memcpy(atr, nss_atr, len);
     *atr_len = len;
 }
@@ -519,18 +519,23 @@  vcard_emul_reader_get_slot(VReader *vreader)
 
 /*
- *  Card ATR's map to physical cards. VCARD_ATR_PREFIX will set appropriate
+ *  Card ATR's map to physical cards. vcard_alloc_atr will set appropriate
  *  historical bytes for any software emulated card. The remaining bytes can be
  *  used to indicate the actual emulator